//Implementierung zu Joystick.h

#include "stdafx.h"
#include "Joystick.h"
#include "ChildView.h"
#include "HexMotion.h"
#include "Resource.h"
#include "IniFile.h"

//statische Variablen
CJoystick*	CJoystick::m_pCJoystick = NULL;				//statischer pointer auf Instanz - mir wird nur schlecht dabei! (heha, 02/09)
bool		CJoystick::JoyThreadAlive = false;
bool		CJoystick::JoyThreadIsRunning = false;
UINT		CJoystick::sleepTime = 0;	// Millisekunden!

//Konstruktor
CJoystick::CJoystick()
{
	TRACE("CJoystick::CJoystick()\n"); //Statusausgabe fr Debugzwecke
	m_pCJoystick=this;
	m_fTranslationFaktor	= CIniFile::m_pCIniFile->GetValueDouble("joystick","translationfactor");
	m_fRotationFaktor		= CIniFile::m_pCIniFile->GetValueDouble("joystick","rotationfactor");
	InitJoystick();
}

//Destruktor
CJoystick::~CJoystick()
{
	TRACE("CJoystick::~CJoystick()\n"); //Statusausgabe fr Debugzwecke
	if(JoyThreadAlive) //added by S.Wei 17.09.2007
	{
		JoyThreadAlive = false;//added by S.Wei 29.08.2007
		joyThread->ExitInstance();//added by S.Wei 29.08.2007
	}
	if(joyThread != NULL)
        joyThread->Delete();
	
	CIniFile::m_pCIniFile->SetValue("joystick","translationfactor",m_fTranslationFaktor);
	CIniFile::m_pCIniFile->SetValue("joystick","rotationfactor",m_fRotationFaktor);
	CIniFile::m_pCIniFile->SetValue("joystick","sleeptime",(int)m_sleepTime);
}

//-- Methoden --
void CJoystick::InitJoystick()
{
	m_sleepTime = (UINT)CIniFile::m_pCIniFile->GetValueInt("joystick", "sleeptime");
	m_suspendTime = (UINT)CIniFile::m_pCIniFile->GetValueInt("joystick", "suspendtime");
	lastErrorJoystickID = NOJOYSTICK;
	joyThread = NULL;
	JoyThreadAlive = true;
	StartJoystick();
}

//            methods for using a joystick
/*following methods added by S.Wei, last change: 03.09.2007*/

void CJoystick::StartJoystick()
{
	if(joyThread == NULL)
	{
		joyThread = AfxBeginThread(&CJoystick::SeparateRoutineForJoystick, this, THREAD_PRIORITY_BELOW_NORMAL);
	}
	else
	{
		joyThread->ResumeThread();
	}
	JoyThreadIsRunning = true;
}

void CJoystick::StopJoystick()
{
	if((joyThread != NULL)&& JoyThreadIsRunning)
	{
		joyThread->SuspendThread();
		JoyThreadIsRunning = false;
	}
}

UINT _cdecl CJoystick::SeparateRoutineForJoystick(LPVOID pParam)
{
	while(JoyThreadAlive)
	{
		static_cast <CJoystick *>(pParam)->JoyStick();
		::SleepEx(sleepTime, TRUE);
	}
	return 0;
}

void CJoystick::JoyStick()
{
	//http://www.chiefdelphi.com/forums/archive/index.php/t-14908.html
	//http://msdn2.microsoft.com/en-us/library/ms709368.aspx

	JOYINFOEX joyInfo;
	MMRESULT err;
	JOYCAPS joycaps;//Eigenschaften, z.B. min/max
	UINT i = joyGetNumDevs();
	
	for(UINT joyID = JOYSTICKID1; joyID<=JOYSTICKID2; joyID++)
	{
		err = joyGetDevCaps(joyID,&joycaps, sizeof(JOYCAPS));
		ZeroMemory(&joyInfo, sizeof(JOYINFOEX));
		joyInfo.dwSize=sizeof(JOYINFOEX);
		joyInfo.dwFlags= JOY_RETURNALL;
		
		err = joyGetPosEx(joyID,&joyInfo);
		if(err!=JOYERR_NOERROR)
		{
			switch(err){
				case MMSYSERR_NODRIVER:
					//no driver installed
					break;
				case MMSYSERR_INVALPARAM:
					//invalid parameter passed
					break;
				case MMSYSERR_BADDEVICEID:
					//joystick ID is invalid
					break;
				case JOYERR_PARMS:
					/* bad parameters */
					break;
				case JOYERR_NOCANDO:
					/* request not completed */
					break;
				case JOYERR_UNPLUGGED:
					//joystick is not pluged in
					break;
			}
			if(lastErrorJoystickID == (joyID + 1)%2)//kein Joystick angeschlossen
			{
				sleepTime = m_suspendTime;
			}
			lastErrorJoystickID = joyID;
		}
		else//use joystick information
		{
			sleepTime = m_sleepTime;
			Joy2Axes4ButtonsOnly(joyInfo, joycaps);
			lastErrorJoystickID = NOJOYSTICK;
		}
	}
}

//following two methods are for a standard Joystick with 2 Axes and 4 Button and a nonconfigurated or configurated colliehead 

//logarithmisch
//added by S.Wei 12.02.2008
void CJoystick::Joy2Axes4ButtonsOnly(JOYINFOEX joyInfo, JOYCAPS joycaps)
{
	float faktor;
	CChildView * pCVWnd = (CChildView *) IP_GETNAMED("m_pWnd", 0, "CChildView");
	//following lines added by S.Wei, last change 29.08.2007
	switch(joyInfo.dwButtonNumber)
	{
		case 0: //Translation - Ebene (keine Taste gedrckt)
			if(joyInfo.dwXpos < JOY_POSITION_X_NULL - JOY_THRESHOLD_X)
			{//Linksbewegung
				faktor = m_fTranslationFaktor * (JOY_POSITION_X_NULL - joyInfo.dwXpos)/(JOY_POSITION_X_NULL - JOY_POSITION_X_MIN);
				pCVWnd->ActionFromInteractionDevice(ID_BTN_X_LEFT, exp(faktor) -1);
			}
			if(joyInfo.dwXpos > JOY_POSITION_X_NULL + JOY_THRESHOLD_X)
			{//Rechtsbewegung
				faktor = m_fTranslationFaktor * (joyInfo.dwXpos - JOY_POSITION_X_NULL)/(JOY_POSITION_X_MAX - JOY_POSITION_X_NULL);
				pCVWnd->ActionFromInteractionDevice(ID_BTN_X_RIGHT, exp(faktor) -1);
			}
			if(joyInfo.dwYpos > JOY_POSITION_Y_NULL + JOY_THRESHOLD_Y)
			{//vor
				faktor = m_fTranslationFaktor * (joyInfo.dwYpos - JOY_POSITION_Y_NULL)/(JOY_POSITION_Y_MAX - JOY_POSITION_Y_NULL);
				pCVWnd->ActionFromInteractionDevice(ID_BTN_Z_FRONT, exp(faktor) -1);
			}
			if(joyInfo.dwYpos < JOY_POSITION_Y_NULL - JOY_THRESHOLD_Y)
			{//zurck
				faktor = m_fTranslationFaktor * (JOY_POSITION_Y_NULL - joyInfo.dwYpos)/(JOY_POSITION_Y_NULL - JOY_POSITION_Y_MIN);
				pCVWnd->ActionFromInteractionDevice(ID_BTN_Z_BACK, exp(faktor) -1);
			}
			break;
		case 1:
			switch(joyInfo.dwButtons)
			{
				case 1://Rotation (Feuertaste)
					if(joyInfo.dwXpos < JOY_POSITION_X_NULL - JOY_THRESHOLD_X)
					{//Linksdrehung
						faktor = m_fRotationFaktor * (JOY_POSITION_X_NULL - joyInfo.dwXpos)/(JOY_POSITION_X_NULL - JOY_POSITION_X_MIN);
						pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_Z_1, exp(faktor) -1);
					}
					if(joyInfo.dwXpos > JOY_POSITION_X_NULL + JOY_THRESHOLD_X)
					{//Rechtsdrehung
						faktor = m_fRotationFaktor * (joyInfo.dwXpos - JOY_POSITION_X_NULL)/(JOY_POSITION_X_MAX - JOY_POSITION_X_NULL);
						pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_Z_2, exp(faktor) -1);
					}
					if(joyInfo.dwYpos > JOY_POSITION_Y_NULL + JOY_THRESHOLD_Y)
					{//vor - Drehung
						faktor = m_fRotationFaktor * (joyInfo.dwYpos - JOY_POSITION_Y_NULL)/(JOY_POSITION_Y_MAX - JOY_POSITION_Y_NULL);
						pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_X_1, exp(faktor) -1);
					}
					if(joyInfo.dwYpos < JOY_POSITION_Y_NULL - JOY_THRESHOLD_Y)
					{//zurck - Drehung
						faktor = m_fRotationFaktor * (JOY_POSITION_Y_NULL - joyInfo.dwYpos)/(JOY_POSITION_Y_NULL - JOY_POSITION_Y_MIN);
						pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_X_2, exp(faktor) -1);
					}
					break;
				case 2://Nullstellung
					pCVWnd->ActionFromInteractionDevice(ID_BTN_CENTER_ZERO);
					break;
				case 4://Aufnahme starten
					pCVWnd->ActionFromInteractionDevice(ID_RECORDER_AUFNAHME);
					break;
				case 8://Aufnahme beenden / Wiedergabe
					if(pCVWnd->getRecording())
					{
						pCVWnd->ActionFromInteractionDevice(ID_RECORDER_STOP);
					}
					else
					{
						pCVWnd->ActionFromInteractionDevice(ID_RECORDER_WIEDERGABE);
					}
					break;
			}
			break;
	}
	
	if(joycaps.wCaps&JOYCAPS_HASPOV)//Colliehead
	{
		if(joycaps.wCaps&JOYCAPS_POV4DIR)
		{
			switch(joyInfo.dwPOV) //Colliehead - bei Voreinstellung
			{
				case JOY_POVFORWARD://hoch (Colliehead hoch)
					pCVWnd->ActionFromInteractionDevice(ID_BTN_Y_UP, m_fTranslationFaktor);
					break;
				case JOY_POVRIGHT://rechts hinter drehen (Colliehead rechts)
					pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_Y_1, m_fRotationFaktor);
					break;
				case JOY_POVBACKWARD://runter (Colliehead runter)
					pCVWnd->ActionFromInteractionDevice(ID_BTN_Y_DOWN, m_fTranslationFaktor);
					break;
				case JOY_POVLEFT://links hinter drehen (Colliehead links)
					pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_Y_2, m_fRotationFaktor);
					break;
			}
		}
	}
	else
	{
		//wenn der Colliehead nicht eingestellt wurde, gibt es folgende Flle (Tastenkombinationen):
		switch(joyInfo.dwButtonNumber)
		{
			case 2: 
				switch(joyInfo.dwButtons)
				{
					case 3://links hinter drehen (Colliehead links)
						pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_Y_2, m_fRotationFaktor);
						break;
				}
				break;
			case 3:
				switch(joyInfo.dwButtons)
				{
					
					case 7://runter (Colliehead runter)
						pCVWnd->ActionFromInteractionDevice(ID_BTN_Y_DOWN, m_fTranslationFaktor);
						break;
					case 11://rechts hinter drehen (Colliehead rechts)
						pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_Y_1, m_fRotationFaktor);
						break;
				}
			case 4:
				switch(joyInfo.dwButtons)
				{
					case 15://hoch (Colliehead hoch)
						pCVWnd->ActionFromInteractionDevice(ID_BTN_Y_UP, m_fTranslationFaktor);
						break;
				}
				break;
		}
	}
		
}

//linear
//added by S.Wei 29.08.2007 as a method since 30.09.2007, last change 12.02.2008
/* deaktiviert bei der Einfhrung der logarithmischen Einteilung 12.02.2008
void CJoystick::Joy2Axes4ButtonsOnly(JOYINFOEX joyInfo, JOYCAPS joycaps)
{
	if (CPlattform::m_pCPlattform == NULL)
	{//Plattform noch nicht initialisiert
		return;
	}

	float faktor;
	CChildView * pCVWnd = (CChildView *) IP_GETNAMED("m_pWnd", 0, "CChildView");
	//following lines added by S.Wei, last change 29.08.2007
	switch(joyInfo.dwButtonNumber)
	{
		case 0: //Translation - Ebene (keine Taste gedrckt)
			if(joyInfo.dwXpos < JOY_POSITION_X_NULL - JOY_THRESHOLD_X)
			{//Linksbewegung
				faktor = m_fTranslationFaktor * (JOY_POSITION_X_NULL - joyInfo.dwXpos)/(JOY_POSITION_X_NULL - JOY_POSITION_X_MIN);
				pCVWnd->ActionFromInteractionDevice(ID_BTN_X_LEFT, faktor);
			}
			if(joyInfo.dwXpos > JOY_POSITION_X_NULL + JOY_THRESHOLD_X)
			{//Rechtsbewegung
				faktor = m_fTranslationFaktor * (joyInfo.dwXpos - JOY_POSITION_X_NULL)/(JOY_POSITION_X_MAX - JOY_POSITION_X_NULL);
				pCVWnd->ActionFromInteractionDevice(ID_BTN_X_RIGHT, faktor);
			}
			if(joyInfo.dwYpos > JOY_POSITION_Y_NULL + JOY_THRESHOLD_Y)
			{//vor
				faktor = m_fTranslationFaktor * (joyInfo.dwYpos - JOY_POSITION_Y_NULL)/(JOY_POSITION_Y_MAX - JOY_POSITION_Y_NULL);
				pCVWnd->ActionFromInteractionDevice(ID_BTN_Z_FRONT, faktor);
			}
			if(joyInfo.dwYpos < JOY_POSITION_Y_NULL - JOY_THRESHOLD_Y)
			{//zurck
				faktor = m_fTranslationFaktor * (JOY_POSITION_Y_NULL - joyInfo.dwYpos)/(JOY_POSITION_Y_NULL - JOY_POSITION_Y_MIN);
				pCVWnd->ActionFromInteractionDevice(ID_BTN_Z_BACK, faktor);
			}
			break;
		case 1:
			switch(joyInfo.dwButtons)
			{
				case 1://Rotation (Feuertaste)
					if(joyInfo.dwXpos < JOY_POSITION_X_NULL - JOY_THRESHOLD_X)
					{//Linksdrehung
						faktor = m_fRotationFaktor * (JOY_POSITION_X_NULL - joyInfo.dwXpos)/(JOY_POSITION_X_NULL - JOY_POSITION_X_MIN);
						pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_Z_1, faktor);
					}
					if(joyInfo.dwXpos > JOY_POSITION_X_NULL + JOY_THRESHOLD_X)
					{//Rechtsdrehung
						faktor = m_fRotationFaktor * (joyInfo.dwXpos - JOY_POSITION_X_NULL)/(JOY_POSITION_X_MAX - JOY_POSITION_X_NULL);
						pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_Z_2, faktor);
					}
					if(joyInfo.dwYpos > JOY_POSITION_Y_NULL + JOY_THRESHOLD_Y)
					{//vor - Drehung
						faktor = m_fRotationFaktor * (joyInfo.dwYpos - JOY_POSITION_Y_NULL)/(JOY_POSITION_Y_MAX - JOY_POSITION_Y_NULL);
						pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_X_1, faktor);
					}
					if(joyInfo.dwYpos < JOY_POSITION_Y_NULL - JOY_THRESHOLD_Y)
					{//zurck - Drehung
						faktor = m_fRotationFaktor * (JOY_POSITION_Y_NULL - joyInfo.dwYpos)/(JOY_POSITION_Y_NULL - JOY_POSITION_Y_MIN);
						pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_X_2, faktor);
					}
					break;
				case 2://Nullstellung
					pCVWnd->ActionFromInteractionDevice(ID_BTN_CENTER_ZERO);
					break;
			}
			break;
	}
	
	if(joycaps.wCaps&JOYCAPS_HASPOV)//Colliehead
	{
		if(joycaps.wCaps&JOYCAPS_POV4DIR)
		{
			switch(joyInfo.dwPOV) //Colliehead - bei Voreinstellung
			{
				case JOY_POVFORWARD://hoch (Colliehead hoch)
					pCVWnd->ActionFromInteractionDevice(ID_BTN_Y_UP);
					break;
				case JOY_POVRIGHT://rechts hinter drehen (Colliehead rechts)
					pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_Y_1);
					break;
				case JOY_POVBACKWARD://runter (Colliehead runter)
					pCVWnd->ActionFromInteractionDevice(ID_BTN_Y_DOWN);
					break;
				case JOY_POVLEFT://links hinter drehen (Colliehead links)
					pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_Y_2);
					break;
			}
		}
	}
	else
	{
		//wenn der Colliehead nicht eingestellt wurde, gibt es folgende Flle (Tastenkombinationen):
		switch(joyInfo.dwButtonNumber)
		{
			case 2: 
				switch(joyInfo.dwButtons)
				{
					case 3://links hinter drehen (Colliehead links)
						pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_Y_2);
						break;
				}
				break;
			case 3:
				switch(joyInfo.dwButtons)
				{
					
					case 7://runter (Colliehead runter)
						pCVWnd->ActionFromInteractionDevice(ID_BTN_Y_DOWN);
						break;
					case 11://rechts hinter drehen (Colliehead rechts)
						pCVWnd->ActionFromInteractionDevice(ID_BTN_TRN_Y_1);
						break;
				}
			case 4:
				switch(joyInfo.dwButtons)
				{
					case 15://hoch (Colliehead hoch)
						pCVWnd->ActionFromInteractionDevice(ID_BTN_Y_UP);
						break;
				}
				break;
		}
	}
		
}
*/

