// OpenGLLight.cpp: Implementierung der Klasse COpenGLLight.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "gl\gl.h"			//OpenGL - Standartbefehle	z.B. glVertex3d(), ...
#include "gl\glu.h"			//OpenGL - Utilities		z.B. gluPerspective(), ...
//#include "gl\glaux.h"		//OpenGL - Hilfsbibliothek	z.B. auxSolidSphere(), ...
#include "IniFile.h"
#include "OpenGLScene.h"
#include "OpenGLLight.h"
#include "OpenGLFunctions.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CList<COpenGLLight*,COpenGLLight*> COpenGLLight::m_pLightList;
//COpenGLLight* COpenGLLight::GetGLLight(int nGLLightNr); //deaktiviert S.Wei 06.08.2007


COpenGLLight::COpenGLLight()
{	char buffer[10];
	m_pLightList.AddTail(this);
	m_nLightNumber=m_pLightList.GetCount()-1;
	_itoa(m_nLightNumber,buffer,10);
	m_bActivated=false;
	m_strName="GL_LIGHT";
	m_strName+=buffer;
	TRACE("COpenGLLight::COpenGLLight() (Nr.%d) =\"%s\" \n",m_nLightNumber,m_strName);
	SetPosition(0.0,0.0,0.0);
//	SetOrientation(0.0,0.0,0.0);

}
COpenGLLight::COpenGLLight(int nGLLightNr)
{	char buffer[10];
	ASSERT(nGLLightNr>=GL_LIGHT0);
	ASSERT(nGLLightNr<=GL_LIGHT7);
	ASSERT(GL_LIGHT0==0x4000);			//nicht unbedingt ntig	
	
	m_pLightList.AddTail(this);
	m_nLightNumber=m_pLightList.GetCount()-1;//liefert 0..7

	m_nLightNumber=nGLLightNr-GL_LIGHT0;
	
	_itoa(m_nLightNumber,buffer,10);
	m_bActivated=false;
	m_strName="GL_LIGHT";
	m_strName+=buffer;
	TRACE("COpenGLLight::COpenGLLight() (Nr.%d) =\"%s\" \n",m_nLightNumber,m_strName);

	SetPosition(0.0,0.0,0.0);
//	SetOrientation(0.0,0.0,0.0);

}


COpenGLLight::~COpenGLLight()
{
	COpenGLLight* pLight;
	POSITION pos=COpenGLLight::m_pLightList.GetHeadPosition();
	POSITION LastPos;
	CString temp;
	while (pos)
	{	LastPos=pos;
		pLight=COpenGLLight::m_pLightList.GetNext(pos);
		
		if (pLight==this)
			COpenGLLight::m_pLightList.RemoveAt(LastPos);
	};
}

void COpenGLLight::SetPosition(float x, float y, float z)
{
	m_fPosition[0]=x;
	m_fPosition[1]=y;
	m_fPosition[2]=z;
}

//void COpenGLLight::SetOrientation(float a, float b, float c)
//{
//	m_fOrientation[0]=a;
//	m_fOrientation[1]=b;
//	m_fOrientation[2]=c;
//}

void COpenGLLight::Activate(bool doit)
{	m_bActivated=doit;
	if (doit) glEnable(GetLightNumberGL());	
    else glDisable(GetLightNumberGL());	
}

void COpenGLLight::ActivateAll(bool doit)
{	if (doit) glEnable(GL_LIGHTING);
    else glDisable(GL_LIGHTING);
}

int COpenGLLight::GetLightNumber()
{	ASSERT(m_nLightNumber>=0);
	ASSERT(m_nLightNumber<=7);
	ASSERT(GL_LIGHT0==0x4000);			//nicht unbedingt ntig
	return m_nLightNumber;				//liefert Zahl zwischen 0...7  (max=8Lights)
}

int COpenGLLight::GetLightNumberGL()
{	ASSERT(m_nLightNumber>=0);
	ASSERT(m_nLightNumber<=7);
	ASSERT(GL_LIGHT0==0x4000);			//nicht unbedingt ntig
	return m_nLightNumber+GL_LIGHT0;	//liefert Zahl zwischen GL_LIGHT0...GL_LIGHT7  (max=8Lights)
}

void COpenGLLight::Init()
{	CIniFile* INI = CIniFile::m_pCIniFile;

	CString LName=m_strName;
	
	float fDefault[4];
	fDefault[0]=0.0f;fDefault[1]=0.0f;fDefault[2]=0.0f;fDefault[3]=1.0f;
	
	if (!INI->IsValue(LName,"active")) INI->SetValue(LName,"active",0,true);
	if (!INI->IsValue(LName,"fixed")) INI->SetValue(LName,"fixed",0,true);
	if (!INI->IsValue(LName,"ambient")) INI->SetValue(LName,"ambient",fDefault,true);
	if (!INI->IsValue(LName,"diffuse")) INI->SetValue(LName,"diffuse",fDefault,true);
	if (!INI->IsValue(LName,"specular")) INI->SetValue(LName,"specular",fDefault,true);
	if (!INI->IsValue(LName,"position")) INI->SetValue(LName,"position",fDefault,true);

	INI->GetValueTrFloat(LName,"ambient",m_fAmbient);
	m_fAmbient[3]=1.0f;
	INI->GetValueTrFloat(LName,"diffuse",m_fDiffuse);
	m_fDiffuse[3]=1.0f;
	INI->GetValueTrFloat(LName,"specular",m_fSpecular);
	m_fSpecular[3]=1.0f;
	
	if (GetLightNumberGL()==GL_LIGHT0)
	{	m_fPosition[0]= COpenGLFunctions::m_pCOpenGLFunctions->m_pScene->m_fViewPosition[0];
		m_fPosition[1]= COpenGLFunctions::m_pCOpenGLFunctions->m_pScene->m_fViewPosition[1];
		m_fPosition[2]= COpenGLFunctions::m_pCOpenGLFunctions->m_pScene->m_fViewPosition[2];
		m_fPosition[3]= 1.0f;
	}else{	
		INI->GetValueTrFloat(LName,"position",m_fPosition);
		m_fPosition[3]= 1.0f;
	}
	
	m_bActivated=(INI->GetValueInt(LName,"active")>0);
	m_bFixed=(INI->GetValueInt(LName,"fixed")>0);
	m_bDrawSourcePoint=(INI->GetValueInt(LName,"drawpoint")>0);
	
	DrawLight();
}

void COpenGLLight::DrawLight()
{	glLightfv(GetLightNumberGL(),GL_AMBIENT,m_fAmbient);	
	glLightfv(GetLightNumberGL(),GL_DIFFUSE,m_fDiffuse);
	glLightfv(GetLightNumberGL(),GL_SPECULAR,m_fSpecular);
	glLightfv(GetLightNumberGL(),GL_POSITION,m_fPosition);
	if (m_bActivated) glEnable(GetLightNumberGL());
}

CString COpenGLLight::GetName(){return m_strName;}


void COpenGLLight::SetLightVar(UINT nType, float* Var)
{
	switch (nType)
	{
	case GL_AMBIENT:	
		m_fAmbient[0]=Var[0];	m_fAmbient[1]=Var[1];	m_fAmbient[2]=Var[2];	m_fAmbient[3]=Var[3]; 
		break;
	case GL_DIFFUSE:	
		m_fDiffuse[0]=Var[0];	m_fDiffuse[1]=Var[1];	m_fDiffuse[2]=Var[2];	m_fDiffuse[3]=Var[3]; 
		break;
	case GL_SPECULAR:	
		m_fSpecular[0]=Var[0];	m_fSpecular[1]=Var[1];	m_fSpecular[2]=Var[2];	m_fSpecular[3]=Var[3]; 
		break;
	case GL_POSITION:	
		m_fPosition[0]=Var[0];	m_fPosition[1]=Var[1];	m_fPosition[2]=Var[2];	m_fPosition[3]=Var[3]; 
		break;
	};
}

bool COpenGLLight::IsActivated()	{return m_bActivated;	}
bool COpenGLLight::IsFixed()		{return m_bFixed;		}

COpenGLLight* COpenGLLight::GetGLLight(int nGLLightNr)
{
	COpenGLLight* pLight;
	POSITION pos=COpenGLLight::m_pLightList.GetHeadPosition();
	CString temp;
	while (pos)
	{	pLight=COpenGLLight::m_pLightList.GetNext(pos);
		if (pLight->GetLightNumberGL()==nGLLightNr) return pLight;
	};
	return NULL;
}


