// Plattform.cpp: Implementierung der Klasse CPlattform.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "gl\gl.h"			//OpenGL - Standartbefehle	z.B. glVertex3d(), ...
#include "gl\glu.h"			//OpenGL - Utilities		z.B. gluPerspective(), ...
#include "gl\glaux.h"		//OpenGL - Hilfsbibliothek	z.B. auxSolidSphere(), ...
#include "Servo.h"
#include "Plattform.h"
#include "OpenGLObject.h"
#include "math.h"
#include "IniFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CPlattform* CPlattform::m_pCPlattform;	//static pointer auf instanz 

CPlattform::CPlattform()
{	CIniFile* INI = CIniFile::m_pCIniFile;	
	m_pCPlattform=this;	
	TRACE("CPlattform::CPlattform()");
	float fTemp[3];
	
	InitPlattformObjects();
// Lage der Plattform im Raum (Szenenkoordinaten)
	INI->GetValueTrFloat("plattform","position",fTemp);
	SetPosition(fTemp);
	INI->GetValueTrFloat("plattform","orientation",fTemp);
	SetOrientation(fTemp);
// Lage der 6 Kugelzapfen
	INI->GetValueTrFloat("plattform","linkpoint1",m_fLinkPointLokal[0]);
	INI->GetValueTrFloat("plattform","linkpoint2",m_fLinkPointLokal[1]);
	INI->GetValueTrFloat("plattform","linkpoint3",m_fLinkPointLokal[2]);
	INI->GetValueTrFloat("plattform","linkpoint4",m_fLinkPointLokal[3]);
	INI->GetValueTrFloat("plattform","linkpoint5",m_fLinkPointLokal[4]);
	INI->GetValueTrFloat("plattform","linkpoint6",m_fLinkPointLokal[5]);
	
	m_dIncrTranslation=INI->GetValueDouble("plattform","incr_translation");
	m_dIncrRotation=INI->GetValueDouble("plattform","incr_rotation");
	
	UpdateLinkPointGlobal();
}

CPlattform::~CPlattform()
{	COpenGLObject* pObject;
	TRACE("CPlattform::~CPlattform()\n");
	
	pObject=COpenGLObject::GetObject("plattform_object");
	if (pObject) delete (pObject);
	m_pCPlattform=NULL;
}

void CPlattform::InitPlattformObjects()
{	COpenGLObject* pObject=NULL;
	CIniFile* INI = CIniFile::m_pCIniFile;	

	CString GeometrieFile;
	float fTemp[4];
	
	if (!COpenGLObject::GetObject("plattform_object"))
	 pObject = new COpenGLObject("plattform_object","active");
	else
		return;//added by S.Wei 29.08.2007
	
	pObject->SetPosition(m_fPosition);			//servo position bernehmen
	pObject->SetOrientation(m_fOrientation);	//servoausrichtung bernehmen
		
	INI->GetValueTrFloat(pObject->GetName(),"position_offset",fTemp);
	pObject->SetPositionOffset(fTemp);			//gehuse positions offset laden
	
	INI->GetValueTrFloat(pObject->GetName(),"orientation_offset",fTemp);
	pObject->SetOrientationOffset(fTemp);		//gehuse ausrichtungs offset laden
	
	if (INI->IsValue(pObject->GetName(),"color"))
	{	INI->GetValueTrFloat(pObject->GetName(),"color",fTemp);
		fTemp[3]=INI->GetValueDouble(pObject->GetName(),"color_alpha");
	}else{
		fTemp[0]=0.1f;fTemp[1]=0.1f;fTemp[2]=0.1f;fTemp[3]=1.0f;
	}
	pObject ->SetColor(fTemp);					//gehuse farbe laden
	
	
	GeometrieFile=INI->GetValue(pObject->GetName(),"geometrie_file");
	if (GeometrieFile.GetLength()>0)
		pObject ->SetGeometrie(GeometrieFile);	//gehuse geometrie laden
	
	pObject->SetShowOrigin(INI->GetValueInt(pObject->GetName(),"show_origin")>0);
	pObject->SetShowGeometrie(INI->GetValueInt(pObject->GetName(),"show_geometrie")>0);

}

void CPlattform::UpdatePlattformObjects()
{	COpenGLObject* pObject=NULL;
	CIniFile* INI = CIniFile::m_pCIniFile;	

	pObject=COpenGLObject::GetObject("plattform_object");
	if (pObject) 
	{	pObject->SetPosition(m_fPosition);				//servo position bernehmen
		pObject->SetOrientation(m_fOrientation);		//gehuse positions offset laden
	};
}

void CPlattform::GetPosition(float *pos) const {
 CServo::Copy3F(pos,m_fPosition);
}

void CPlattform::GetOrientation(float *orientation) const{
 CServo::Copy3F(orientation,m_fOrientation);
}

void CPlattform::SetPosition(const float *pos) {
 CServo::Copy3F(m_fPosition,pos);
 UpdatePlattformObjects();
 UpdateLinkPointGlobal();
}

void CPlattform::SetPosition(float x,float y,float z) {
 float v[3]={x,y,z};
 SetPosition(v);
}

void CPlattform::SetOrientation(const float *orientation) {
 CServo::Copy3F(m_fOrientation,orientation);
 UpdatePlattformObjects();
 UpdateLinkPointGlobal();
}

void CPlattform::SetOrientation(float a,float b,float c) {
 float v[3]={a,b,c};
 SetOrientation(v);
}

void CPlattform::GetLinkPointLokal(int P,float pos[3]) const {
 CServo::Copy3F(pos,m_fLinkPointLokal[P-1]);
}

void CPlattform::GetLinkPointGlobal(int P,float pos[3]) const {
 CServo::Copy3F(pos,m_fLinkPointGlobal[P-1]);
}
void CPlattform::SetLinkPointLokal(int P,const float v[3]) {
 CServo::Copy3F(m_fLinkPointGlobal[P-1],v);
 UpdateLinkPointGlobal();
}
void CPlattform::SetLinkPointLokal(int P,float x,float y,float z) {
 float v[3]={x,y,z};
 SetLinkPointLokal(P,v);
}
// Berechnen der Lage der 6 Kugelzapfen im Raum (Szenenkoordinaten)
void CPlattform::UpdateLinkPointGlobal() {
 float A[3][3];
 CServo::Drehmatrix(m_fOrientation,A);
 for (int n=0;n<6;n++)
  CServo::MatrixTrafo(A,m_fLinkPointLokal[n],m_fPosition,m_fLinkPointGlobal[n]);
}
