//Implementierung zu Script.h zum Aufzeichnen und Wiedergeben von Bewegungen

#include "stdafx.h"
#include "Script.h"
#include "HexMotion.h"
#include "ChildView.h"
#include "IniFile.h"
//folgende zwei Bibliotheken fr die Zeitnahme notwendig
#include <sys/timeb.h>
#include <time.h>

//statische Variablen
CScript*	CScript::m_pCScript = NULL;
bool		CScript::ScriptThreadAlive = false;
bool		CScript::ScriptThreadIsRunning = false;
UINT		CScript::timeNextStep = 0;
__timeb64	CScript::startzeit;
UINT		CScript::averageStepTime = 100;//100 Millisekunden

//Konstruktor
CScript::CScript(CString path, bool readonly)
{
	m_pCScript = this;
	scriptThread = NULL;
	this->path = path;
	if(this->readonly = readonly)
	{
		this->readFromFile();
		_ftime64(&startzeit); //Zeit nehmen
		ScriptThreadAlive = true;
		this->StartScript();
	}
}

//Destruktor
CScript::~CScript()
{
	if(this->readonly)
	{
		if(scriptThread != NULL)
		{
			scriptThread->SuspendThread();
			scriptThread->ExitInstance();
			scriptThread->Delete();
		}
		if(ScriptThreadAlive)
		{
			ScriptThreadAlive = false;
		}
	}
	else
	{
		this->writeInFile();
	}
	CScript::m_pCScript = NULL;
}

//-- Methoden --
void CScript::StartScript()
{
	if(scriptThread == NULL)
	{
		scriptThread = AfxBeginThread(&CScript::SeparateRoutineForScript, this, THREAD_PRIORITY_NORMAL);
	}
	else
	{
		scriptThread->ResumeThread();
	}
	ScriptThreadIsRunning = true;
}

void CScript::StopScript()
{
	if((scriptThread != NULL)&& ScriptThreadIsRunning)
	{
		scriptThread->SuspendThread();
		ScriptThreadIsRunning = false;
	}
}

UINT _cdecl CScript::SeparateRoutineForScript(LPVOID pParam)
{
	struct __timeb64	ttime;
	UINT	difftime;
	while(ScriptThreadAlive)
	{
		static_cast <CScript *>(pParam)->Scripting();
		_ftime64(&ttime);//Zeit nehmen
		difftime = (UINT) (ttime.time*1000 + ttime.millitm - startzeit.time*1000 - startzeit.millitm); 
		if(difftime < timeNextStep)
		{
			timeNextStep -= difftime; //kein Offset ntig, da die seit der letzten Zeitnahme vergangene Zeit unter 1 ms liegt
			::SleepEx(timeNextStep, TRUE);
		}
	}
	delete(CScript::m_pCScript); //die Klasse wird gelscht
	return TRUE;
}

//Methode zum Ausfhren der gewollten Bewegungen
void CScript::Scripting()
{
	if(m_ActionList.GetCount() > 0)
	{//mindestens noch ein Befehl vorhanden
		CChildView * pCVWnd = (CChildView *) IP_GETNAMED("m_pWnd", 0, "CChildView");
		if(pCVWnd == NULL)
			return;//keine Verbindung zu den Modellen

		HEXAPOD_SCRIPTPART scriptAction = (HEXAPOD_SCRIPTPART) m_ActionList.RemoveHead();
		
		if(m_ActionList.GetCount() > 0)
		{//mindestens noch ein weiterer Befehl vorhanden
			//Wartezeit einstellen
			HEXAPOD_SCRIPTPART scriptAction2 = (HEXAPOD_SCRIPTPART) m_ActionList.GetHead();
			if(!(scriptAction2.bart & BEW_ART_NOWAIT))
			{
				timeNextStep = scriptAction2.starttime;
			}
		}
		if(scriptAction.bart & BEW_ART_FUNCTION)
		{//Bewegung anhand einer Funktion
			
			return;
		}
		int steps = 1;
		if(scriptAction.bart & BEW_ART_LOW)
		{//langsame Bewegung
			steps = 1 + (scriptAction.endtime - scriptAction.starttime)/averageStepTime;
		}
		for(int i = 0; i < steps; i++)
		{
			if(scriptAction.bart & BEW_ART_LOW)
			{
				if(scriptAction.bart & BEW_ART_RELATIV)
				{
					pCVWnd->movePlattform(scriptAction.x, scriptAction.y, scriptAction.z, scriptAction.rx, scriptAction.ry, scriptAction.rz, steps);
				}
				else
				{//absolut
					pCVWnd->setPlattform(scriptAction.x, scriptAction.y, scriptAction.z, scriptAction.rx, scriptAction.ry, scriptAction.rz, steps - i);
				}
			}
			else
			{
				if(scriptAction.bart & BEW_ART_RELATIV)
				{
					pCVWnd->movePlattform(scriptAction.x, scriptAction.y, scriptAction.z, scriptAction.rx, scriptAction.ry, scriptAction.rz);
				}
				else
				{//absolut
					pCVWnd->setPlattform(scriptAction.x, scriptAction.y, scriptAction.z, scriptAction.rx, scriptAction.ry, scriptAction.rz);
				}
			}
			pCVWnd->SendMessage(WM_KEYUP); //diese Nachricht bewirkt die Aktualisierung der Anzeige //added by S.Wei, 24.04.2008
		}
		// Statusanzeige
		struct __timeb64	ttime; _ftime64(&ttime);//Zeit nehmen
		double d = (double) ttime.time - startzeit.time; d = d + (ttime.millitm - startzeit.millitm)/1000.0;
		CString zeitstempel; zeitstempel.Format("Wiedergabe gestartet %0.3fs", (double)d);
		((CMainFrame *)IP_GET("m_pCMainFrame"))->m_wndStatusBar->SetWindowText(zeitstempel);		
	}
	else
	{
		//alle Befehle sind abgearbeitet, der Prozess wird beendet
		scriptThread = NULL;
		ScriptThreadAlive = false;
		//Statusanzeige
		((CMainFrame *)IP_GET("m_pCMainFrame"))->m_wndStatusBar->SetWindowText("Wiedergabe beendet");
	}
}

void CScript::functionScripting(HEXAPOD_SCRIPTPART hsp)
{
	//muss noch implementiert werden
}


void CScript::OnAction(float *pos, float *rot, UINT starttime/*= 0*/, UINT endtime/*= 0*/, UINT bart/* = BEW_ART_NOWAIT*/)
{
	OnAction(pos[0], pos[1], pos[2], rot[0], rot[1], rot[2], starttime, endtime, bart);
}

void CScript::OnAction(float x, float y, float z, float rx, float ry, float rz, UINT starttime/* = 0*/, UINT endtime/*= 0*/, UINT bart/* = BEW_ART_NOWAIT*/)
{
	if(!readonly)
	{
		HEXAPOD_SCRIPTPART hsp;
		hsp.x = x;
		hsp.y = y;
		hsp.z = z;
		hsp.rx = rx;
		hsp.ry = ry;
		hsp.rz = rz;
		hsp.starttime = starttime;
		hsp.endtime = endtime;
		if(starttime >= endtime)
			bart = bart & (!BEW_ART_LOW);
		hsp.bart = bart;
		//Befehl bernehmen
		POSITION p = m_ActionList.AddTail(hsp);
	}
	else
	{
		return;
	}
}


void CScript::writeNextCommand(CString key)
{
	if(m_ActionList.GetCount() > 0)
	{
		HEXAPOD_SCRIPTPART hsp = (HEXAPOD_SCRIPTPART)m_ActionList.RemoveHead();
		CIniFile::m_pCIniFile->SetValue(key,"x",hsp.x);
		CIniFile::m_pCIniFile->SetValue(key,"y",hsp.y);
		CIniFile::m_pCIniFile->SetValue(key,"z",hsp.z);
		CIniFile::m_pCIniFile->SetValue(key,"rx",hsp.rx);
		CIniFile::m_pCIniFile->SetValue(key,"ry",hsp.ry);
		CIniFile::m_pCIniFile->SetValue(key,"rz",hsp.rz);
		CIniFile::m_pCIniFile->SetValue(key,"starttime",(int)hsp.starttime);
		CIniFile::m_pCIniFile->SetValue(key,"endtime",(int)hsp.endtime);
		CIniFile::m_pCIniFile->SetValue(key,"movement_type",(int)hsp.bart);
	}
}

void CScript::readNextCommand(CString key)
{
	HEXAPOD_SCRIPTPART hsp;
	hsp.x			= CIniFile::m_pCIniFile->GetValueDouble	(key,"x");
	hsp.y			= CIniFile::m_pCIniFile->GetValueDouble	(key,"y");
	hsp.z			= CIniFile::m_pCIniFile->GetValueDouble	(key,"z");
	hsp.rx			= CIniFile::m_pCIniFile->GetValueDouble	(key,"rx");
	hsp.ry			= CIniFile::m_pCIniFile->GetValueDouble	(key,"ry");
	hsp.rz			= CIniFile::m_pCIniFile->GetValueDouble	(key,"rz");
	hsp.starttime	= (UINT)	CIniFile::m_pCIniFile->GetValueInt		(key,"starttime");
	hsp.endtime		= (UINT)	CIniFile::m_pCIniFile->GetValueInt		(key,"endtime");
	hsp.bart		= (UINT)	CIniFile::m_pCIniFile->GetValueInt		(key,"movement_type");
	hsp.function	=			CIniFile::m_pCIniFile->GetValue			(key, "fuction");
	m_ActionList.AddTail(hsp);
}

void CScript::readFromFile()
{
	//geffneten Zustand sichern
	CString oldPath = CIniFile::m_pCIniFile->GetPath();
	CIniFile::m_pCIniFile->WriteFile();

	//neue Datei ffnen
	CIniFile::m_pCIniFile->SetPath(path);
	CIniFile::m_pCIniFile->Reset();
	if(!CIniFile::m_pCIniFile->ReadFile())
		return;

	bool noend = true;
	for(int i = 1; noend; i++)
	{
		CString temp;
		temp.Format("%d",i);
		if(CIniFile::m_pCIniFile->IsKey(temp))
		{
			this->readNextCommand(temp);
		}
		else
		{
			noend = false;
		}
	}

	//vorherige Datei laden
	CIniFile::m_pCIniFile->Reset();
	CIniFile::m_pCIniFile->SetPath(oldPath);
	CIniFile::m_pCIniFile->ReadFile();
}

void CScript::writeInFile()
{
	//geffneten Zustand sichern
	CString oldPath = CIniFile::m_pCIniFile->GetPath();
	CIniFile::m_pCIniFile->WriteFile();

	//neue Datei ffnen
	CIniFile::m_pCIniFile->SetPath(path);
	CIniFile::m_pCIniFile->Reset();
	//CIniFile::m_pCIniFile->ReadFile();

	int n = m_ActionList.GetCount();
	for(int i = 1; m_ActionList.GetCount() > 0; i++)
	{
		TRACE("Speichern: %d von %d\n", i, n);
		CString temp;
		temp.Format("%d",i);
		this->writeNextCommand(temp);
	}
	CIniFile::m_pCIniFile->WriteFile();

	//vorherige Datei laden
	CIniFile::m_pCIniFile->Reset();
	CIniFile::m_pCIniFile->SetPath(oldPath);
	CIniFile::m_pCIniFile->ReadFile();
}