//Script.h - Headerdatei fr Script.cpp
/**	Diese Klasse CScript erlaubt das Aufzeichnen und Abspielen von Bewegungen des Hexapods
	Sie ist auf die Verwendung durch Hexapod/Hexmotion zugeschnitten worden.
	Entwickelt: S.Wei 02.01.2008

	Zu beachten: Entsprechend der Implementierung greift die Klasse auf die Methode ActionFromInteractionDevice der Klasse CChildView zu.
*/
//angelehnt an CJoystick von S.Wei vom 20.11.2007

#pragma once
#include "stdafx.h"

#include "resource.h"   // Hauptsymbole
#include "SerialCom.h"	// Hinzugefgt von der Klassenansicht
#include "IniFile.h"	// Hinzugefgt von der Klassenansicht
#include "MainFrm.h"
#include "ChildView.h"
//folgende zwei Bibliotheken fr die Zeitnahme notwendig
#include <sys/timeb.h>
#include <time.h>


#define BEW_ART_FUZZY		0x01	//a = const; !linear
#define BEW_ART_LOW			0x02	//langsames Anfahren der Endposition; !sofortiges Anfahren	(Bedingung: Endzeit > Startzeit)
#define BEW_ART_NOWAIT		0x04	//anschlieend sofortiges Ausfhren des nchsten Befehls	(entspricht: Startzeit = 0)
#define BEW_ART_RELATIV		0x08	//realative Bewegung; !absolut
#define BEW_ART_FUNCTION	0x10	//eine Funktion beschreibt die Bewegung (noch nicht implementiert)


typedef struct scriptPartForHexapod
{
	float	x;
	float	y;
	float	z;
	float	rx;
	float	ry;
	float	rz;
	UINT	starttime;
	UINT	endtime;
	UINT	bart;
	CString	function;
} HEXAPOD_SCRIPTPART, FAR *LPHEXAPOD_SCRIPTPART;

class CScript
{
//Konstruktoren/Destruktor
public:
	CScript(CString path, bool readonly = true);
	virtual ~CScript();

//Attribute
public:
	static	CScript*	m_pCScript;
	static	bool		ScriptThreadAlive;
	static	bool		ScriptThreadIsRunning;
	static	UINT		timeNextStep;
	static	__timeb64	startzeit;
	static	UINT		averageStepTime;

private:
	CWinThread		*scriptThread;
	CString			path;
	CString			konfigPath;
	bool			readonly;

	CList <HEXAPOD_SCRIPTPART,HEXAPOD_SCRIPTPART&> m_ActionList;
	
//Methoden
public:
	void StartScript();
	void StopScript();
	static UINT _cdecl SeparateRoutineForScript(LPVOID pParam);
	void Scripting();

	void OnAction(float *pos, float *rot, UINT starttime = 0, UINT endtime= 0, UINT bart = BEW_ART_NOWAIT);
	void OnAction(float x, float y,	float z, float rx, float ry, float rz, UINT starttime = 0, UINT endtime= 0, UINT bart = BEW_ART_NOWAIT);
	
private:
	void functionScripting(HEXAPOD_SCRIPTPART hsp);
	void writeNextCommand(CString key);
	void readNextCommand(CString key);
	void readFromFile();
	void writeInFile();
};
