#include "stdafx.h"
#include "gl\gl.h"			//OpenGL - Standartbefehle	z.B. glVertex3d(), ...
#include "gl\glu.h"			//OpenGL - Utilities		z.B. gluPerspective(), ...
#include "gl\glaux.h"		//OpenGL - Hilfsbibliothek	z.B. auxSolidSphere(), ...
#include "math.h"
#include "HexMotion.h"
#include "IniFile.h"
#include "resource.h"
#include "Plattform.h"
#include "OpenGLObject.h"
#include "OpenGLScene.h"
#include "OpenGLFunctions.h"
#include "Servo.h"
#include <shlwapi.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CList<CServo*,CServo*> CServo::m_pServoList;

const float CServo::piover180 = 0.017453292519943F;

CServo::CServo()
{
	CIniFile* INI = CIniFile::m_pCIniFile;	
	
	char buffer[30];
	float fTemp[3]={0};
	
	m_pServoList.AddTail(this);
	m_nServoNumber=m_pServoList.GetCount();

	sprintf(buffer, "servo%d",m_nServoNumber);
	m_strName=buffer;
	TRACE("CServo::CServo() (ServoNr%d=\"%s\")\n",m_nServoNumber,m_strName);

	InitServoObjects();
	
	INI->GetValueTrFloat(GetName(),"position",fTemp);
	SetPosition(fTemp);
	
	INI->GetValueTrFloat(GetName(),"orientation",fTemp);
	SetOrientation(fTemp);

	int nTemp=INI->GetValueInt(GetName(),"usedangle");	// = 0 keiner 1="linker" 2="rechter" Servo-Arm
	nTemp = (nTemp<=1) ? 1 : 2 ;
	SetUsedAngle(nTemp);

	SetStartAngle(INI->GetValueDouble(GetName(),"startangle"));
	SetAngle(INI->GetValueDouble(GetName(),"angle"));
	SetArmLength(INI->GetValueDouble(GetName(),"armlength"));
	SetLinkLength(INI->GetValueDouble(GetName(),"linklength"));	
	SetAngleCorrectionFactor(INI->GetValueDouble(GetName(),"anglefactor"));	
	m_fAngleMinIncrements=10;	// Servoanschlag links
	m_fAngleMaxIncrements=200;	// Servoanschlag rechts

	m_bLinked=false;

	UpdateLinkPoint();
	
	CalcConnection();
}

CServo::~CServo()
{	COpenGLObject* pObject;
	CServo* pServo;
	POSITION pos=CServo::m_pServoList.GetHeadPosition();
	POSITION LastPos;
	CString strTemp;
	char buffer[30];
	while (pos)
	{	LastPos=pos;
		pServo=CServo::m_pServoList.GetNext(pos);
		if (pServo==this)
		{	
			TRACE("CServo::~CServo() (ServoNr%d=\"%s\")\n",pServo->GetServoNumber(),pServo->GetName());
			CServo::m_pServoList.RemoveAt(LastPos);

			sprintf(buffer, "servo%d_housing",m_nServoNumber);	
			pObject=COpenGLObject::GetObject(buffer);
			
			if (pObject) delete (pObject);
			sprintf(buffer, "servo%d_angle",m_nServoNumber);	
			
			pObject=COpenGLObject::GetObject(buffer);
			if (pObject) delete (pObject);
		}
	};
}

//void CServo::SetPosition(float x,float y,float z) {
// float v[3]={x,y,z};
// SetPosition(v);
//}


//void CServo::SetOrientation(float a,float b,float c) {
// float v[3]={a,b,c};
// SetOrientation(v);
//}

void CServo::SetAngle(float fAngle) {
 m_fAngle=fAngle;	
//	char buffer[30];
 COpenGLObject* pObject;
//	sprintf(buffer, "servo%d_angle",m_nServoNumber);	
	
 float m_fDrawAngle=180.0f-m_fAngle-m_fStartAngle;
	
 pObject=COpenGLObject::GetObject(GetName()+"_angle");		//Servo-Hebelarm suchen
 if (pObject) pObject->SetOrientation(m_fOrientation[0],m_fOrientation[1],m_fOrientation[2]+m_fDrawAngle);
 UpdateLinkPoint();
}

CServo* CServo::GetServo(int nServoNr) {
 CServo* pServo;
 POSITION pos=CServo::m_pServoList.GetHeadPosition();
 CString temp;
 while (pos) {
  pServo=CServo::m_pServoList.GetNext(pos);
  if (pServo->GetServoNumber()==nServoNr) return pServo;
 };
 return NULL;
}

CServo* CServo::GetServo(CString strServoName) {
 CServo* pServo;
 POSITION pos=CServo::m_pServoList.GetHeadPosition();
 CString temp;
 while (pos) {
  pServo=CServo::m_pServoList.GetNext(pos);
  if (pServo->GetName()==strServoName) return pServo;
 };
 return NULL;
}

void CServo::UpdateServoObjects()
{	COpenGLObject* pObject;
	CIniFile* INI = CIniFile::m_pCIniFile;	

	pObject=COpenGLObject::GetObject(GetName()+"_housing");
	if (pObject) 
	{	pObject->SetPosition(m_fPosition);				//servo position bernehmen
		pObject->SetOrientation(m_fOrientation);		//gehuse positions offset laden
	}
	pObject=COpenGLObject::GetObject(GetName()+"_angle");
	if (pObject)
	{	pObject->SetPosition(m_fPosition);				//servo position bernehmen
		pObject->SetOrientation(m_fOrientation);		//gehuse positions offset laden
	}
}

void CServo::InitServoObjects()
{	COpenGLObject* pObject;
	CIniFile* INI = CIniFile::m_pCIniFile;	

	CString GeometrieFile;
	float fTemp[4];
	CString strObject;

//-------- GEHUSE ERZEUGEN / INITIALISIEREN ---------------------------------	

	strObject=GetName()+"_housing";
	if (!COpenGLObject::GetObject(strObject))
		pObject = new COpenGLObject(strObject,"active");
	
	pObject->SetPosition(m_fPosition);			//servo position bernehmen
	pObject->SetOrientation(m_fOrientation);	//servoausrichtung bernehmen
		
	INI->GetValueTrFloat(pObject->GetName(),"position_offset",fTemp);
	pObject->SetPositionOffset(fTemp);			//gehuse positions offset laden
	
	INI->GetValueTrFloat(pObject->GetName(),"orientation_offset",fTemp);
	pObject->SetOrientationOffset(fTemp);		//gehuse ausrichtungs offset laden
	
	if (INI->IsValue(pObject->GetName(),"color"))
	{	INI->GetValueTrFloat(pObject->GetName(),"color",fTemp);
		fTemp[3]=INI->GetValueDouble(pObject->GetName(),"color_alpha");
	}else{
		fTemp[0]=0.1f;fTemp[1]=0.1f;fTemp[2]=0.1f;fTemp[3]=1.0f;
	}
	pObject ->SetColor(fTemp);					//gehuse farbe laden
	
	
	GeometrieFile=INI->GetValue(pObject->GetName(),"geometrie_file");
	if (GeometrieFile.GetLength()>0)
		pObject ->SetGeometrie(GeometrieFile);	//gehuse geometrie laden
	
	pObject->SetShowOrigin(INI->GetValueInt(pObject->GetName(),"show_origin")>0);
	pObject->SetShowGeometrie(INI->GetValueInt(pObject->GetName(),"show_geometrie")>0);

//-------- HEBELARM ERZEUGEN / INITIALISIEREN --------------------------------	

	strObject=GetName()+"_angle";
	if (!COpenGLObject::GetObject(strObject))
		pObject = new COpenGLObject(strObject,"active");
	
	pObject->SetPosition(m_fPosition);			//servo position bernehmen
	pObject->SetOrientation(m_fOrientation);	//servoausrichtung bernehmen
		
	INI->GetValueTrFloat(pObject->GetName(),"position_offset",fTemp);
	pObject->SetPositionOffset(fTemp);			//gehuse positions offset laden
	
	INI->GetValueTrFloat(pObject->GetName(),"orientation_offset",fTemp);
	pObject->SetOrientationOffset(fTemp);		//gehuse ausrichtungs offset laden
	
	if (INI->IsValue(pObject->GetName(),"color"))
	{	INI->GetValueTrFloat(pObject->GetName(),"color",fTemp);
		fTemp[3]=INI->GetValueDouble(pObject->GetName(),"color_alpha");
	}else{
		fTemp[0]=0.1f;fTemp[1]=0.1f;fTemp[2]=0.1f;fTemp[3]=1.0f;
	}
	pObject ->SetColor(fTemp);					//gehuse farbe laden
	
	
	GeometrieFile=INI->GetValue(pObject->GetName(),"geometrie_file");
	if (GeometrieFile.GetLength()>0)
		pObject ->SetGeometrie(GeometrieFile);	//gehuse geometrie laden
	
	pObject->SetShowOrigin(INI->GetValueInt(pObject->GetName(),"show_origin")>0);
	pObject->SetShowGeometrie(INI->GetValueInt(pObject->GetName(),"show_geometrie")>0);

}

void CServo::DestroyServoObjects() {
 COpenGLObject::m_pObjectList.RemoveAll();
}

void CServo::UpdateLinkPoint() {
 float winkel=(180-GetAngle()-GetStartAngle())*piover180;
	
 m_fLinkPointLokal[0]=m_fArmLength*cos(winkel);
 m_fLinkPointLokal[1]=m_fArmLength*sin(winkel);
 m_fLinkPointLokal[2]=0.0f;

 MatrixTrafo(m_A,m_fLinkPointLokal,m_fPosition,m_fLinkPointGlobal);
//	TransformL2G(m_fLinkPointLokal,m_fPosition,m_fOrientation,m_fLinkPointGlobal);
}

// Generiert die systemangepasste 3x3-Drehmatrix (Determinante = 1) aus den 3 Winkeln im Gradma
void CServo::Drehmatrix(const float orient[3], float A[3][3]) {
 float s[3], c[3];		// Sinsse und Cosinsse
 for (int i=0; i<3; i++) {
  float a=orient[i]*piover180;	// (Winkel im Bogenma)
  s[i]=sin(a);
  c[i]=cos(a);
 }
/*     Z(T)     x        X(T)     x        Y(T)      =             Drehmatrix(T)
|  Cz  Sz  0  |   |  1   0   0  |   |  Cy  0  -Sy  |   | CzCy-SzSxSy  SzCx -CzSy-SzSxCy |
| -Sz  Cz  0  | x |  0   Cx -Sx | x |  0   1   0   | = |-SzCy-CzSxSy  CzCx  SzSy-CzSxCy |
|  0   0   1  |   |  0   Sx  Cx |   |  Sy  0   Cy  |   |        CxSy    Sx         CxCy |
*/
 A[0][0]= c[2]*c[0]-s[2]*s[1]*s[0];
 A[0][1]=-s[2]*c[0]-c[2]*s[1]*s[0];
 A[0][2]=                c[1]*s[0];
 A[1][0]=           s[2]*c[1]     ;
 A[1][1]=           c[2]*c[1]     ;
 A[1][2]=                s[1]     ;
 A[2][0]=-c[2]*s[0]-s[2]*s[1]*c[0];
 A[2][1]= s[2]*s[0]-c[2]*s[1]*c[0];
 A[2][2]=                c[1]*c[0];
}

// Drehung (mit A) und  Verschiebung (um b) eines Vektors (x) nach (y) entsprechend y = A*x + b
void CServo::MatrixTrafo(const float A[3][3],const float x[3], const float b[3], float y[3]) {
 for (int i=0; i<3; i++) {
  y[i]=A[i][0]*x[0]
      +A[i][1]*x[1]
      +A[i][2]*x[2]
      +b[i];
 }
}

// Berechnet u.a. den bentigten Winkel fr den Servo
bool CServo::CalcConnection() {	
	CPlattform* pPlattform;
	m_bLinked=false;
	
	float	m_dSPg[3];	// Servo Position (mittelpunkt der CP-Bahn)	(globales KS)
	float	m_dSA[3];	// Servo Ausrichtung
	float	m_dPPg[3];	// Plattform-LinkPoint				(globales KS)
	float	m_dPPl[3];	// Plattform-LinkPoint				(lokales Servo-KS)
	float	m_dCP[4][3];	// ConnectionPoint (CP)				(globales KS)
	
	float	m_dSR;		// Servo-Hebelarm
	float	m_dPR;		// Stablnge

	float	m_dServoAngle;
	float	fTemp[3];
	
	pPlattform=CPlattform::m_pCPlattform;
	if (!pPlattform) return false;
	
	pPlattform->GetLinkPointGlobal(m_nServoNumber,fTemp);
	setPlattformPointGlobal(fTemp);

	Copy3F(m_dPPg,fTemp);
	Copy3F(m_dSPg,m_fPosition);
	Copy3F(m_dSA,m_fOrientation);

	m_dSR=m_fArmLength;
		
	m_dPR=m_fLinkLength;
//	m_dPR=20.0;

//	bool m_bNoLink=false;
	
	float dCPR;										//Radius der CP-Kreisbahn (=Schnittkreis in Servo-XY-Ebene)
	float dCPP[2];										//Mittelpunkt des Schnittkreises 
	float AngleA=0.0;
	float AngleX=0.0;	
	float AngleB=0.0;

		TransformG2L(m_dPPg,m_dSPg,m_dSA,m_dPPl);
		setPlattformPointLokal(m_dPPl);

		if (fabs(m_dPPl[2])<fabs(m_dPR))					// Wenn der Abstand des LinkPoints von der
														// ServoEbene grer als die Stablnge ist,
														// gibt es keinen Schnittpunkt.
		{	dCPR=sqrt(sqr(m_dPR)-sqr(m_dPPl[2]));	// CP-Bahn Radius auf ServoEbene (ebener Phytagoras)
			dCPP[0]=m_dPPl[0];							// X-Position des CP-Bahn-Mittelpunktes
			dCPP[1]=m_dPPl[1];							// Y-Position des CP-Bahn-Mittelpunktes
		}else
		{	TRACE("PlattformLinkPoint hat zu groen Abstand von ServoEbene (Pz=%d)\n",m_dPPl[2]);
			dCPR=0.0;
			m_bLinked=false;
			return false;
		};

 ZeroMemory(m_dCP,sizeof(m_dCP));
		
		if ((m_dPPl[0]!=0.0)&&(m_dPPl[1]!=0.0))
		{	//Annahme Px<>0 und Py<>0 -> Rechnung sollte ohne Div durch Null klappen
			if (!GetCircleIntersection(m_dSR,dCPP,dCPR,m_dCP[0],m_dCP[1]))
			{	m_bLinked=false;
				return false;
			};
		}
		else
		{
			return false;
		}
		
		if		((m_dCP[0][0]>=0.0)&&(m_dCP[0][1]>=0.0)) AngleA=  0+asin(m_dCP[0][1]/m_dSR)/piover180;
		else if ((m_dCP[0][0]< 0.0)&&(m_dCP[0][1]>=0.0)) AngleA=180-asin(m_dCP[0][1]/m_dSR)/piover180;
		else if ((m_dCP[0][0]< 0.0)&&(m_dCP[0][1]< 0.0)) AngleA=180-asin(m_dCP[0][1]/m_dSR)/piover180;	
		else if ((m_dCP[0][0]>=0.0)&&(m_dCP[0][1]< 0.0)) AngleA=360+asin(m_dCP[0][1]/m_dSR)/piover180;	
		ASSERT((AngleA<=360.0)&&(AngleA>=0.0));
		
		float dist=sqrt(sqr(m_dPPl[0])+sqr(m_dPPl[1]));
		if		((m_dPPl[0]>=0.0)&&(m_dPPl[1]>=0.0)) AngleX=  0+asin(m_dPPl[1]/dist)/piover180;
		else if ((m_dPPl[0]< 0.0)&&(m_dPPl[1]>=0.0)) AngleX=180-asin(m_dPPl[1]/dist)/piover180;
		else if ((m_dPPl[0]< 0.0)&&(m_dPPl[1]< 0.0)) AngleX=180-asin(m_dPPl[1]/dist)/piover180;	
		else if ((m_dPPl[0]>=0.0)&&(m_dPPl[1]< 0.0)) AngleX=360+asin(m_dPPl[1]/dist)/piover180;	
		ASSERT((AngleX<=360.0)&&(AngleX>=0.0));
		
		
		if		((m_dCP[1][0]>=0.0)&&(m_dCP[1][1]>=0.0)) AngleB=  0+asin(m_dCP[1][1]/m_dSR)/piover180;
		else if ((m_dCP[1][0]< 0.0)&&(m_dCP[1][1]>=0.0)) AngleB=180-asin(m_dCP[1][1]/m_dSR)/piover180;
		else if ((m_dCP[1][0]< 0.0)&&(m_dCP[1][1]< 0.0)) AngleB=180-asin(m_dCP[1][1]/m_dSR)/piover180;	
		else if ((m_dCP[1][0]>=0.0)&&(m_dCP[1][1]< 0.0)) AngleB=360+asin(m_dCP[1][1]/m_dSR)/piover180;	
		ASSERT((AngleB<=360.0)&&(AngleB>=0.0));
		
		//Ziel: AngleA ist "rechts"von AngleX
	
		float dTemp;
		if ((AngleX<__min(AngleA,AngleB))||(AngleX>__max(AngleA,AngleB)))
		{	if(AngleA<AngleB)
			{	dTemp=m_dCP[0][0];m_dCP[0][0]=m_dCP[1][0];m_dCP[1][0]=dTemp;
				dTemp=m_dCP[0][1];m_dCP[0][1]=m_dCP[1][1];m_dCP[1][1]=dTemp;
				dTemp=AngleB;AngleB=AngleA;AngleA=dTemp;
				ASSERT(AngleA>=AngleB);
			}
		}
		else //if ((AngleX>AngleA)&&(AngleX<AngleB))
		{	if(AngleA>AngleB)
			{	dTemp=m_dCP[0][0];m_dCP[0][0]=m_dCP[1][0];m_dCP[1][0]=dTemp;
				dTemp=m_dCP[0][1];m_dCP[0][1]=m_dCP[1][1];m_dCP[1][1]=dTemp;
				dTemp=AngleB;AngleB=AngleA;AngleA=dTemp;
				ASSERT(AngleA<=AngleB);
			}
		};
		
		if (m_nUsedAngle==1)
			m_dServoAngle=180.0F-AngleA-m_fStartAngle;
		else
			m_dServoAngle=180.0F-AngleB-m_fStartAngle;
		
		if (m_dServoAngle<0.0) m_dServoAngle+=360.0;
		if (m_dServoAngle>=360.0) m_dServoAngle-=360.0;


		SetAngle(m_dServoAngle);

/*
	glPushMatrix();										
		glTranslated(m_dSPg[0],m_dSPg[1],m_dSPg[2]);	//ServoMittelpunkt ist Ursprung vom Servo-KS
		glRotated(m_dSA[0],0.0,1.0,0.0);				
		glRotated(m_dSA[1],-1.0,0.0,0.0);
		glRotated(m_dSA[2],0.0,0.0,1.0);

		glLineWidth(4.0);
		glPointSize(8.0);
		glColor3d(1.0,0.0,0.0);
		if (m_nUsedAngle==1)
		{	glBegin(GL_POINTS);
				glVertex3d(m_dCP[0][0],m_dCP[0][1],m_dCP[0][2]);
			glEnd();
			glBegin(GL_LINES);
				glVertex3d(0.0,0.0,0.0);
				glVertex3d(m_dCP[0][0],m_dCP[0][1],m_dCP[0][2]);
				glVertex3d(m_dCP[0][0],m_dCP[0][1],m_dCP[0][2]);
				glVertex3d(m_dPPl[0],m_dPPl[1],m_dPPl[2]);
			glEnd();
		}else
		{	glBegin(GL_POINTS);
				glVertex3d(m_dCP[1][0],m_dCP[1][1],m_dCP[1][2]);
			glEnd();
			glBegin(GL_LINES);
				glVertex3d(0.0,0.0,0.0);
				glVertex3d(m_dCP[1][0],m_dCP[1][1],m_dCP[1][2]);	
				glVertex3d(m_dCP[1][0],m_dCP[1][1],m_dCP[1][2]);
				glVertex3d(m_dPPl[0],m_dPPl[1],m_dPPl[2]);
			glEnd();
		};
	glPopMatrix();
*/	
	SetLinked(true);
	
	return true;	
}

// Rcktransformation (wird eigentlich nur bentigt, weil die Winkelberechnungsroutine so gestrickt ist)
// Mit this.m_A knnte man auch das Gleichungssystem lsen oder eine inverse Matrix ansetzen
void CServo::TransformG2L(const float y[3], const float dZpos_g[3], const float dZorient[3], float x[3]) {
	// y = Punktkoordinaten (global)
	// x = Punktkoordinaten (lokal)
	// dZpos_g = Ursprung des lokalen KS (globel)
	// dZorient_l = Ausrichtung des lokalen KS im globalen KS 
 int i;
 float s[3],c[3];	// Sinus, Kosinus
 for (i=0; i<3; i++) {
  s[i]=sin(dZorient[i]*piover180);
  c[i]=cos(dZorient[i]*piover180);
 }
 float F[3], G[3];	// Zwischenergebnisvektoren
// Translation
 F[0]=y[0]-dZpos_g[0];
 F[1]=y[1]-dZpos_g[1];
 F[2]=y[2]-dZpos_g[2];
// Drehung um Y-Achse
 G[0]= F[0]*c[0]     -F[2]*s[0];
 G[1]=           F[1]          ;
 G[2]= F[0]*s[0]     +F[2]*c[0];
// Drehung um X-Achse
 F[0]= G[0]                    ;
 F[1]=      G[1]*c[1]-G[2]*s[1];
 F[2]=      G[1]*s[1]+G[2]*c[1];
// Drehung um Z-Achse
 x[0]= F[0]*c[2]+F[1]*s[2]     ;
 x[1]=-F[0]*s[2]+F[1]*c[2]     ;
 x[2]=                     F[2];
}

bool CServo::GetCircleIntersection(float dRA, const float dPB[2], float dRB, float dP1[3], float dP2[3])
{	//Kreis1:			=   x               + y               = dRA	(ServoHebelBahn)
	//Kreis2:			=   (x-xm)          + (y-ym)          = dRB	(CP-Bahn)
	//
	//Kreis1			=	x               + y               = dRA
	//Kreis2			=	x - 2xm*x + xm + y - 2ym*y + ym = dRB
	//
	//Kreis1 - Kreis2	=	     2xm*x - xm      + 2ym*y - ym = dRA-dRB
	//
	//					     dRA-dRB+xm+ym       2my*y
	//				  x'=	-------------------  -  -------  =  Ca - Cb*y 
	//								2xm				  2xm
	//
	//Kreis1			=	x'				+ y		- dRA	= 0
	//Kreis1			=  (Ca - Cb*y)     + y        - dRA  = 0
	//
	//Kreis1			=   Ca - 2*Cb*Ca*y+ (Cb+1)*y - dRA	= 0
	//
	//					          2*Cb*Ca          Ca-dRA
	//Kreis1			=   y - ---------- * y + ---------  = 0  = y - p*y + q = 0
	//					           CB+1            CB+1
	//
	//				  
	//				 y0 = -p/2 +/- sqrt ( p/4 - q )
	
	float Ca,Cb;
	float p,q;
	float SP[4][2];

	Ca=(sqr(dRA)-sqr(dRB)+sqr(dPB[0])+sqr(dPB[1]))/(2*dPB[0]);
	Cb=(dPB[1]/dPB[0]);
	
	p=(-2*Cb*Ca)/(Cb*Cb+1);
	q=(Ca*Ca-dRA*dRA)/(Cb*Cb+1);

		if (((p*p)/4-q)<=0){
			//auerhalb des Aktionsradius
			TRACE("No Intersection! (dPB[0]=%f dPB[1]=%f)\n",dPB[0],dPB[1]);
			//following line added by S.Wei, 20.12.2007
			MessageBeep(MB_ICONEXCLAMATION);
			Beep(440,10);//Sound - kratzendes Warngerusch
			return false;
		}else{
				SP[0][1]=(-1*p)/2-sqrt(sqr(p)/4-q);
				SP[1][1]=(-1*p)/2+sqrt(sqr(p)/4-q);
	
				SP[2][1]=SP[0][1];
				SP[3][1]=SP[1][1];
				
				SP[0][0]=sqrt(sqr(dRA)-sqr(SP[0][1]));
				SP[1][0]=sqrt(sqr(dRA)-sqr(SP[1][1]));
				
				SP[2][0]=-SP[0][0];
				SP[3][0]=-SP[1][0];
				
				// Die Lsung ergibt 4 Resultate
				// -> 2x beim Lsen der Quadratischen Gleichung
				// -> 2x wegen der Wurzelfunktion beim Einsetzen in die Kreisgleichung
				// Es sind aber praktisch nur 2 Resultate mglich
				
				//Suche der minimalen Abweichungen von der Kugelfunktion 
				//zum Finden der beiden richtigen Schnittpunkte
				//-> Einsetzen in Kugelgleichung  dX+dY+dZ=R -> dX+dY+dZ-R= MIN!

				//Kreis2      =  (x-xm)          + (y-ym)          = dRB	(CP-Bahn)
				//Abweichung  =  (SP[n][0]-dPB[0])  + (SP[n][1]-dPB[1]) - dRB  ->  MIN !!!
				float abw[4];
				
				abw[0]=fabs(sqr(SP[0][0]-dPB[0])+sqr(SP[0][1]-dPB[1])-sqr(dRB));
				abw[1]=fabs(sqr(SP[1][0]-dPB[0])+sqr(SP[1][1]-dPB[1])-sqr(dRB));
				abw[2]=fabs(sqr(SP[2][0]-dPB[0])+sqr(SP[2][1]-dPB[1])-sqr(dRB));
				abw[3]=fabs(sqr(SP[3][0]-dPB[0])+sqr(SP[3][1]-dPB[1])-sqr(dRB));

				int v1,v2,n;

				int min[2]= {10,10};
				
				for(n=0;n<=1;n++){
					v1=0;												//abw[v1] ist veRgleichsbasis
					if (min[0]==v1) v1=1;								//kann erst fr n=1 passieren
					v2=v1+1;
					if (min[0]==v2) v2+=1;								//kann erst fr n=1 passieren
					min[n]=v1;
					do
					{	if (abw[v1]>abw[v2])
						{	min[n]=v2;									//abw[v2] ist jetzt minimum 
							v1=v2;				
						};
						v2+=1;
						if (v2==min[0]) v2+=1;
					}while (v2<=3);
				};
											
				dP1[0]=SP[min[0]][0];
				dP1[1]=SP[min[0]][1];
				dP1[2]=0.0;
				dP2[0]=SP[min[1]][0];
				dP2[1]=SP[min[1]][1];
				dP2[2]=0.0;
		};
		return true;
}



/* seit 21.12.2007 nur noch zum Servo testen Verwendung; Aufgaben werden durch die Methoden GetMoveString() und MoveAllServos() erledigt*/
void CServo::MoveRealServo() const {	
	int nRealServoIncrements=(int)(m_fAngleMinIncrements+(m_fAngleCorrectionFactor/90*m_fAngle));
	
	if (m_fAngle>270)		nRealServoIncrements=(int)m_fAngleMinIncrements;
	else if (m_fAngle>180)	nRealServoIncrements=(int)m_fAngleMaxIncrements;
	


	if (nRealServoIncrements<m_fAngleMinIncrements) nRealServoIncrements=(int)m_fAngleMinIncrements;
	if (nRealServoIncrements>m_fAngleMaxIncrements) nRealServoIncrements=(int)m_fAngleMaxIncrements;
	
 if (gApp.m_SerialCom.IsConnected()) {
  char buffer[8];
  int n = wnsprintfA(buffer,elemof(buffer),"(m%d%03d)",m_nServoNumber-1,nRealServoIncrements);
  gApp.m_SerialCom.WriteBytes(buffer,n);
 }
}

//Befehl fr das Bewegen des Servos, Grundlagen entstammen SerialCom.cpp
CString CServo::GetMoveString() const{
 TCHAR buf[8];
	int nRealServoIncrements=(int)(m_fAngleMinIncrements+(m_fAngleCorrectionFactor/90*m_fAngle));
	
	if (m_fAngle>270)		nRealServoIncrements=(int)m_fAngleMinIncrements;
	else if (m_fAngle>180)	nRealServoIncrements=(int)m_fAngleMaxIncrements;
	if (nRealServoIncrements<m_fAngleMinIncrements) nRealServoIncrements=(int)m_fAngleMinIncrements;
	if (nRealServoIncrements>m_fAngleMaxIncrements) nRealServoIncrements=(int)m_fAngleMaxIncrements;

 wnsprintf(buf,elemof(buf),_T("(m%d%03d)"),m_nServoNumber-1,nRealServoIncrements);
 return buf;
}

bool CServo::MoveAllServos() {
//folgende Zeilen wurden am 21.12.2007 aus MainFrm.cpp hierher ausgelagert
 if (!gApp.m_SerialCom.IsConnected()) return false;
 CString befehl;
 for (int n=1; n<=6; n++) {
  CServo *Servo = GetServo(n);
  if (Servo && Servo->CalcConnection()) befehl += Servo->GetMoveString();	//added by S.Wei 03.09.2007
  else return false;
 }	/*http://msdn2.microsoft.com/en-us/library/awkwbzyc(VS.80).aspx*/
 return gApp.m_SerialCom.WriteBytes((const char*)befehl,befehl.GetLength());
}
